#include "../../../include/http/pages/peerdetailspage.h"
#include "../../../include/stringfunctions.h"

#ifdef XMEM
	#include <xmem.h>
#endif

const std::string PeerDetailsPage::GeneratePage(const std::string &method, const std::map<std::string,std::string> &queryvars)
{
	std::string content="";
	int identityid=0;
	std::string identityidstr="";
	std::string name;
	std::string publickey;
	std::string messagetrust;
	std::string trustlisttrust;
	std::string keypart="";
	std::string lastseen="";
	std::string dateadded="";
	std::string addedmethod="";

	if(queryvars.find("identityid")!=queryvars.end() && (*queryvars.find("identityid")).second!="")
	{
		identityidstr=(*queryvars.find("identityid")).second;
		StringFunctions::Convert((*queryvars.find("identityid")).second,identityid);
	}

	if(identityid!=0 && queryvars.find("formaction")!=queryvars.end() && (*queryvars.find("formaction")).second=="deletemessages")
	{
		SQLite3DB::Statement del=m_db->Prepare("DELETE FROM tblMessage WHERE IdentityID=?;");
		del.Bind(0,identityid);
		del.Step();
	}

	SQLite3DB::Statement st=m_db->Prepare("SELECT Name,PublicKey,DateAdded,LastSeen,AddedMethod FROM tblIdentity WHERE IdentityID=?;");
	st.Bind(0,identityid);
	st.Step();

	content+="<table>";
	if(st.RowReturned())
	{
		st.ResultText(0,name);
		st.ResultText(1,publickey);
		st.ResultText(2,dateadded);
		st.ResultText(3,lastseen);
		st.ResultText(4,addedmethod);

		content+="<tr><td>Name</td><td>"+SanitizeOutput(name)+"</td></tr>";
		content+="<tr><td>Public Key</td><td class=\"smaller\">"+SanitizeOutput(publickey)+"</td></tr>";
		content+="<tr><td>Date Added</td><td>"+dateadded+"</td></tr>";
		content+="<tr><td>Last Seen</td><td>"+lastseen+"</td></tr>";
		content+="<tr><td>Added Method</td><td class=\"smaller\">"+SanitizeOutput(addedmethod)+"</td></tr>";
	}

	// get message count posted by this identity
	st=m_db->Prepare("SELECT COUNT(MessageID) FROM tblMessage WHERE IdentityID=?;");
	st.Bind(0,identityid);
	st.Step();

	if(st.RowReturned())
	{
		std::string messagecountstr="0";
		st.ResultText(0,messagecountstr);
		content+="<tr>";
		content+="<td>Message Count</td>";
		content+="<td>"+messagecountstr;
		content+="&nbsp;&nbsp;<form name=\"frmdeletemessages\" method=\"POST\">";
		content+="<input type=\"hidden\" name=\"identityid\" value=\""+identityidstr+"\">";
		content+="<input type=\"hidden\" name=\"formaction\" value=\"deletemessages\">";
		content+="<input type=\"submit\" value=\"Delete Messages\">";
		content+="</form>";
		content+="</td>";
		content+="</tr>";
	}

	content+="</table>";


	st=m_db->Prepare("SELECT Name,PublicKey,MessageTrust,TrustListTrust,tblIdentity.IdentityID,tblPeerTrust.MessageTrustComment,tblPeerTrust.TrustListTrustComment FROM tblPeerTrust INNER JOIN tblIdentity ON tblPeerTrust.TargetIdentityID=tblIdentity.IdentityID WHERE tblPeerTrust.IdentityID=? ORDER BY Name COLLATE NOCASE;");
	st.Bind(0,identityid);
	st.Step();

	content+="<table>";
	content+="<tr><th colspan=\"3\">";
	content+="Trust List of this identity";
	content+="</th></tr>";
	content+="<tr><td></td><th>Message Trust</th><th>Message Comment</th><th>Trust List Trust</th><th>Trust Comment</th></tr>";
	while(st.RowReturned())
	{
		std::string thisid="";
		std::string messagetrustcomment="";
		std::string trustlisttrustcomment="";

		st.ResultText(0,name);
		st.ResultText(1,publickey);
		st.ResultText(2,messagetrust);
		st.ResultText(3,trustlisttrust);
		st.ResultText(4,thisid);
		st.ResultText(5,messagetrustcomment);
		st.ResultText(6,trustlisttrustcomment);

		if(publickey.size()>8)
		{
			keypart=publickey.substr(3,5);
		}

		content+="<tr>";
		content+="<td><a href=\"peerdetails.htm?identityid="+thisid+"\">"+SanitizeOutput(name+keypart)+"...</a></td>";
		content+="<td "+GetClassString(messagetrust)+">"+messagetrust+"</td>";
		content+="<td>"+SanitizeOutput(messagetrustcomment)+"</td>";
		content+="<td "+GetClassString(trustlisttrust)+">"+trustlisttrust+"</td>";
		content+="<td>"+SanitizeOutput(trustlisttrustcomment)+"</td>";
		content+="</tr>\r\n";

		st.Step();
	}

	st=m_db->Prepare("SELECT Name,PublicKey,MessageTrust,TrustListTrust,tblIdentity.IdentityID,tblPeerTrust.MessageTrustComment,tblPeerTrust.TrustListTrustComment FROM tblPeerTrust INNER JOIN tblIdentity ON tblPeerTrust.IdentityID=tblIdentity.IdentityID WHERE tblPeerTrust.TargetIdentityID=? ORDER BY Name COLLATE NOCASE;");
	st.Bind(0,identityid);
	st.Step();

	content+="<tr><th colspan=\"3\">";
	content+="Trust of this identity from other identities";
	content+="</th></tr>";
	content+="<tr><td></td><th>Message Trust</th><th>Message Comment</th><th>Trust List Trust</th><th>Trust Comment</th></tr>";
	while(st.RowReturned())
	{
		std::string thisid="";
		std::string messagetrustcomment="";
		std::string trustlisttrustcomment="";

		st.ResultText(0,name);
		st.ResultText(1,publickey);
		st.ResultText(2,messagetrust);
		st.ResultText(3,trustlisttrust);
		st.ResultText(4,thisid);
		st.ResultText(5,messagetrustcomment);
		st.ResultText(6,trustlisttrustcomment);
		
		if(publickey.size()>8)
		{
			keypart=publickey.substr(3,5);
		}

		content+="<tr>";
		content+="<td><a href=\"peerdetails.htm?identityid="+thisid+"\">"+SanitizeOutput(name+keypart)+"...</a></td>";
		content+="<td "+GetClassString(messagetrust)+">"+messagetrust+"</td>";
		content+="<td>"+SanitizeOutput(messagetrustcomment)+"</td>";
		content+="<td "+GetClassString(trustlisttrust)+">"+trustlisttrust+"</td>";
		content+="<td>"+SanitizeOutput(trustlisttrustcomment)+"</td>";
		content+="</tr>";

		st.Step();
	}
	content+="</table>";

	return "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n"+StringFunctions::Replace(m_template,"[CONTENT]",content);
}

const std::string PeerDetailsPage::GetClassString(const std::string &trustlevel)
{
	int tempint=0;
	std::string tempstr;

	StringFunctions::Convert(trustlevel,tempint);
	tempint/=10;
	StringFunctions::Convert(tempint,tempstr);

	if(trustlevel!="")
	{
		return "class=\"trust"+tempstr+"\"";
	}
	else
	{
		return "";
	}
}

const bool PeerDetailsPage::WillHandleURI(const std::string &uri)
{
	if(uri.find("peerdetails.")!=std::string::npos)
	{
		return true;
	}
	else
	{
		return false;
	}
}
